{
 ***** BEGIN LICENSE BLOCK *****
 The contents of this file are subject to the Mozilla Public
 License Version 1.1 (the "License"); you may not use this file
 except in compliance with the License. You may obtain a copy of
 the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS
 IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 implied. See the License for the specific language governing
 rights and limitations under the License.

 The Original Code is AnalyseSpectrale code.

 The Initial Developer of the Original Code is Arthur Torossian.
 Portions created by Arthur Torossian are Copyright (C) 2008 Arthur Torossian,
 living in Paris, 20 rue Sibuet, in 2008. All Rights Reserved.

 Original Author: Arthur Torossian (arthur.torossian@free.fr)

 Contributor(s): None

 Alternatively, the contents of this file may be used under the terms of the
 GNU Lesser General Public License (the  "LGPL License"), in which case the
 provisions of the LGPL License are applicable instead of those above.
 If you wish to allow use of your version of this file only under the terms
 of the LGPL License and not to allow others to use your version of this file
 under the MPL, indicate your decision by deleting  the provisions above and
 replace  them with the notice and other provisions required by the LGPL
 License.  If you do not delete the provisions above, a recipient may use
 your version of this file under either the MPL or the LGPL License.

 For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html
 ***** END LICENSE BLOCK ***** */
}

unit UFFT_iterative;

interface
uses
  sysUtils
  , Math
  , UTools
  , UFFT_iterative_tabs
  , UMyMMSystem
  ;
{
n  x b
0 1  1
1 2  2
2 4  4
3 8  8
4 16 10
5 32 20
6 64 40
7 128 80
8 256 100
9 512  200
10 1024 400
11 2048  800
12 4096  1000
}
type
  CFFT_iterative =
    class
      Tampon_num : integer ;
      Tampon : Twindows_4096 ;
      a : Twindows_4096 ;
      module_min, module_max : double ;

      AA : TAAwindows_4096 ;
      module : Twindows_4096 ;

      window : Pwindows_4096 ;
      constructor create ;
      destructor destroy ; override ;

      procedure init ;
      procedure FFT_2048_2048( src : PSon_S16 ) ;
    end ;

procedure gen_bit_reversal_8 ;
procedure gen_bit_reversal_1024 ;

procedure gen_w_4096 ;
procedure gen_bit_reversal_4096 ;
procedure gen_hamming_4096 ;
procedure gen_hanning_4096 ;
procedure gen_Blackman_4096 ;
procedure gen_Blackman_Harris_4096 ;
procedure gen_Blackman_Nuttall_4096 ;
implementation
function bit_reversal_4096( a : integer  ) : integer ;
begin
  result := 0 ;
  if (a and $1) = $1 then result := result  or $800 ;
  if (a and $2) = $2 then result := result  or $400 ;
  if (a and $4) = $4 then result := result  or $200 ;
  if (a and $8) = $8 then result := result  or $100 ;
  if (a and $10) = $10 then result := result  or $80 ;
  if (a and $20) = $20 then result := result  or $40 ;
  if (a and $40) = $40 then result := result  or $20 ;
  if (a and $80) = $80 then result := result  or $10 ;
  if (a and $100) = $100 then result := result  or $8 ;
  if (a and $200) = $200 then result := result  or $4 ;
  if (a and $400) = $400 then result := result  or $2 ;
  if (a and $800) = $800 then result := result  or $1 ;
  //if (a and $400) = $400 then result := result  or $1 ;
end ;


function bit_reversal_1024( a : integer  ) : integer ;
begin
  result := 0 ;
  if (a and $1) = $1 then result := result  or $200 ;
  if (a and $2) = $2 then result := result  or $100 ;
  if (a and $4) = $4 then result := result  or $80 ;
  if (a and $8) = $8 then result := result  or $40 ;
  if (a and $10) = $10 then result := result  or $20 ;
  if (a and $20) = $20 then result := result  or $10 ;
  if (a and $40) = $40 then result := result  or $8 ;
  if (a and $80) = $80 then result := result  or $4 ;
  if (a and $100) = $100 then result := result  or $2 ;
  if (a and $200) = $200 then result := result  or $1 ;
  //if (a and $400) = $400 then result := result  or $1 ;
end ;

constructor CFFT_iterative.create ;
begin
  window := @tab_Blackman_Nuttall_4096 ;
end ;

destructor CFFT_iterative.destroy ;
begin
  inherited destroy ;
end ;

procedure CFFT_iterative.init ;
var
  i : integer ;
begin
  Tampon_num := 0 ;
  for i:= 0 to 4095 do
    begin
      Tampon[ i ] := 0 ;
    end ;
end ;

procedure CFFT_iterative.FFT_2048_2048( src : PSon_S16 ) ;
var
  i,s, j, k, km2 :integer ;
  re, im,  c, gd, fenetre : double ;
  m, m2 : integer ;
  wm, w , t, u, x, mcomp: TComplex ;
begin
  module_min := 1e30 ;
  module_max := -1e30 ;

  if Tampon_num = 0 then
    begin
      // XXXX0000
      j := 2048 ;
      for i := 0 to 2047 do
        begin
          fenetre := window[ j ] ;
          gd :=  ( src^[i].g/65536 + src^[i].d/65536 );
          Tampon[ i ] := gd ;
          a[ j ] := fenetre * gd ;
          inc(j);
        end ;

      j := 2048 ;
      for i := 0 to 2047 do
        begin
          a[ i ] := window[ i ] * Tampon[ j ] ;
          inc(j);
        end ;

      Tampon_num := 1 ;
    end
  else
    begin
      // 0000XXXX
      j := 2048 ;
      for i := 0 to 2047 do
        begin
          fenetre := window[ j ] ;
          gd :=  ( src^[i].g/65536 + src^[i].d/65536 ) ;
          Tampon[ j ] := gd ;
          a[ j ] := fenetre * gd ;
          inc(j);
        end ;

      for i := 0 to 2047 do
        begin
          a[ i ] :=window[ i ] * Tampon[ i ] ;
        end ;

      Tampon_num := 0 ;
    end ;

  for i := 0 to 4095 do
    begin
       k := tab_bit_reversal_4096[i] ;
       AA[ k ].re := a[i] ;
       AA[ k ].im := 0 ;
    end ;

  m := 1 ;
  for s := 1 to  12 do
    begin
      m2 := m ;
      m := m2 * 2 ;
      wm := Tab_w_4096[ s-1 ] ;
      //wm.re := cos( c / m   ) ;
      //wm.im :=  sin( c / m  ) ;
      w.re := 1 ; w.im := 0 ;
      for j := 0 to m2 - 1 do
        begin
          k := j ;
          while k < 4096  do
            begin
              km2 := k + m2 ;
              x := AA[ km2 ] ;

              t.re := w.re * x.re - w.im * x.im  ;
              t.im := w.re * x.im + w.im * x.re  ;
              u := AA[ k  ] ;
              AA[ k  ].re := u.re + t.re ;
              AA[ k  ].im := u.im + t.im ;

              AA[ km2  ].re := u.re - t.re ;
              AA[ km2  ].im := u.im - t.im ;


              inc(k, m) ;
            end ;
          mcomp := w ;
          w.re := mcomp.re * wm.re - mcomp.im * wm.im ;
          w.im := mcomp.re * wm.im + mcomp.im * wm.re ;

        end ;
    end ;


  for i := 0 to 4095 do
    begin
       re := AA[i].re;
       im := AA[i].im ;
       c := log10( re* re + im * im ) ;
       if module_min > c then module_min := c ;
       if module_max < c then module_max := c ;
       Module[i] :=  c
    end ;

end ;


function bit_reversal_8( a : integer  ) : integer ;
begin
  result := 0 ;
  if (a and $1) = $1 then result := result  or $4 ;
  if (a and $2) = $2 then result := result  or $2 ;
  if (a and $4) = $4 then result := result  or $1 ;
end ;


procedure gen_bit_reversal_1024 ;
var
  f : textfile ;
  i : integer ;
  s : string ;
begin
  assignfile( f , 'bit_reversal_1024.txt' );
  rewrite(f) ;
  writeln(f, '    tab_bit_reversal_1024 :Tbit_reversal_1024 = (' ) ;
  s := '        ' ;
  for i := 0 to 1022 do
    begin
      s := s + inttostr( bit_reversal_1024( i  ) )+',' ;
      if i mod 10 = 9 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  s := s + inttostr( bit_reversal_1024( 1023  ) ) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;

procedure gen_bit_reversal_8 ;
var
  f : textfile ;
  i : integer ;
  s : string ;
begin
  assignfile( f , 'bit_reversal_8.txt' );
  rewrite(f) ;
  writeln(f, '    tab_bit_reversal_8 :Tbit_reversal_8 = (' ) ;
  s := '        ' ;
  for i := 0 to 6 do
    begin
      s := s + inttostr( bit_reversal_8( i  ) )+',' ;
      if i mod 10 = 9 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  s := s + inttostr( bit_reversal_8( 7  ) ) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;

procedure gen_bit_reversal_4096 ;
var
  f : textfile ;
  i : integer ;
  s : string ;
begin
  assignfile( f , 'bit_reversal_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_bit_reversal_4096 :Tbit_reversal_4096 = (' ) ;
  s := '        ' ;
  for i := 0 to 4094 do
    begin
      s := s + inttostr( bit_reversal_4096( i  ) )+',' ;
      if i mod 10 = 9 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  s := s + inttostr( bit_reversal_4096( 4095  ) ) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;





procedure gen_w_4096 ;
var
  f : textfile ;
  i : integer ;
  m : extended ;
  c : Tcomplex ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'w_4096.txt' );
  rewrite(f) ;
  writeln(f, '    Tab_w_4096 : Tw_4096 = (' ) ;
  s := '        ' ;
  for i := 1 to 11 do
    begin
      m := IntPower( 2 ,i) ;
      c.re := cos( 2* Pi /m  ) ;
      c.im := sin( 2* Pi /m  ) ;

      s := s + '(re:'+formatfloat('0.0000000',(c.re))+'; im:'+formatfloat('0.0000000',(c.im))+'),'  ;
      writeln(f,s) ;
      s := '        ' ;

    end ;
      m := IntPower( 2 ,12) ;
      c.re := cos( 2* Pi /m  ) ;
      c.im := sin( 2* Pi /m  ) ;

  s := s + '(re:'+formatfloat('0.0000000',(c.re))+'; im:'+formatfloat('0.0000000',(c.im))+')'  ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;



procedure gen_hamming_4096 ;
var
  f : textfile ;
  i : integer ;
  x : double ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'hamming_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_hamming_4096 :Twindows_4096 = (' ) ;
  s := '        ' ;
  for i := 0 to 4094 do
    begin
      x := 0.53836  -  0.46164 * cos( (2 * Pi * i )/(4095) ) ;
      s := s + formatfloat('0.0000000',(x))+',' ;
      if i mod 5 = 4 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  x := 0.53836  -  0.46164 * cos( (2 * Pi * 4095 )/(4095) ) ;
  s := s + formatfloat('0.0000000',(x)) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;


procedure gen_hanning_4096 ;
var
  f : textfile ;
  i : integer ;
  x : double ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'hanning_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_hanning_4096 :Twindows_4096 = (' ) ;
  s := '        ' ;
  for i := 0 to 4094 do
    begin
      x := 0.5  -  0.5 * cos( (2 * Pi * i )/(4096) ) ;
      s := s + formatfloat('0.0000000',(x))+',' ;
      if i mod 5 = 4 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  x := 0.5  -  0.5 * cos( (2 * Pi * 4095 )/(4096) ) ;
  s := s + formatfloat('0.0000000',(x)) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;

  closefile(f);
end ;


procedure gen_Blackman_4096 ;
var
  f : textfile ;
  i : integer ;
  x,w : double ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'Blackman_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_Blackman_4096 :Twindows_4096 = (' ) ;
  s := '        ' ;
  w := 2 * Pi / 4096 ;
  for i := 0 to 4094 do
    begin
      x := 0.42 - 0.50 * cos( w*i ) + 0.08 * cos(2 * w * i) ;
      s := s + formatfloat('0.0000000',(x))+',' ;
      if i mod 5 = 4 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  x := 0.42 - 0.50 * cos( w*4095 ) + 0.08 * cos(2 * w * 4095)  ;
  s := s + formatfloat('0.0000000',(x)) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;
  closefile(f);
end ;

procedure gen_Blackman_Harris_4096 ;
var
  f : textfile ;
  i : integer ;
  x,w : double ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'Blackman_Harris_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_Blackman_Harris_4096 :Twindows_4096 = (' ) ;
  s := '        ' ;
  w := 2 * Pi / 4096 ;
  for i := 0 to 4094 do
    begin
      x := 0.422323 - 0.49755*cos(w*i) + 0.07922*cos(2*w*i) ;
      s := s + formatfloat('0.0000000',(x))+',' ;
      if i mod 5 = 4 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  x := 0.422323 - 0.49755*cos(w*4095) + 0.07922*cos(2*w*4095) ;
  s := s + formatfloat('0.0000000',(x)) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;
  closefile(f);
end ;

procedure gen_Blackman_Nuttall_4096 ;
var
  f : textfile ;
  i : integer ;
  x,w : double ;
  s : string ;
begin
  DecimalSeparator := '.' ;
  assignfile( f , 'Blackman_Nuttall_4096.txt' );
  rewrite(f) ;
  writeln(f, '    tab_Blackman_Nuttall_4096 :Twindows_4096 = (' ) ;
  s := '        ' ;
  w := 2 * Pi / 4096 ;
  for i := 0 to 4094 do
    begin
      x := 0.3635819 - 0.4891775*cos(w*i) + 0.1365995*cos(2*w*i) - 0.0106411*cos(3*w*i) ;
      s := s + formatfloat('0.0000000',(x))+',' ;
      if i mod 5 = 4 then
        begin
          writeln(f,s) ;
          s := '        ' ;
        end ;
    end ;
  x := 0.3635819 - 0.4891775*cos(w*4095) + 0.1365995*cos(2*w*4095) - 0.0106411*cos(3*w*4095) ;
  s := s + formatfloat('0.0000000',(x)) ;
  writeln(f,s) ;
  writeln(f, '  );' ) ;
  closefile(f);
end ;

end.
