{
 ***** BEGIN LICENSE BLOCK *****
 The contents of this file are subject to the Mozilla Public
 License Version 1.1 (the "License"); you may not use this file
 except in compliance with the License. You may obtain a copy of
 the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS
 IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 implied. See the License for the specific language governing
 rights and limitations under the License.

 The Original Code is AnalyseSpectrale code.

 The Initial Developer of the Original Code is Arthur Torossian.
 Portions created by Arthur Torossian are Copyright (C) 2008 Arthur Torossian,
 living in Paris, 20 rue Sibuet, in 2008. All Rights Reserved.

 Original Author: Arthur Torossian (arthur.torossian@free.fr)

 Contributor(s): None

 Alternatively, the contents of this file may be used under the terms of the
 GNU Lesser General Public License (the  "LGPL License"), in which case the
 provisions of the LGPL License are applicable instead of those above.
 If you wish to allow use of your version of this file only under the terms
 of the LGPL License and not to allow others to use your version of this file
 under the MPL, indicate your decision by deleting  the provisions above and
 replace  them with the notice and other provisions required by the LGPL
 License.  If you do not delete the provisions above, a recipient may use
 your version of this file under either the MPL or the LGPL License.

 For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html
 ***** END LICENSE BLOCK ***** */
}

unit UIAnalyseSpectrale;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls
  , Spin
  , math
  , GDIPAPI
  , GDIPOBJ
  , UTools
  , UMyMMSystem
  , UWaveRecorder, ExtCtrls
  , UFFT_iterative
  , UFFT_iterative_tabs
  ;

const
  Freq_plage_dec : array[0..3] of integer = (-1, 0, 1, 2 ) ;
  Freq_plage_Val : array[0..3] of double = ( 5000, 10000, 20000, 40000 ) ;
  remanance_defaut : array[0..11] of double = (
                0, 0.1, 0.2, 0.3, 0.4 , 0.5 ,0.6, 0.7, 0.8, 0.9, 0.95 ,0.99 ) ;
  Titre = 'Analyse spectale par AT ' ;

type
  TForm1 = class(TForm)
    tool: TPanel;
    CB_Pilotes: TComboBox;
    command: TPanel;
    Bt_start: TButton;
    Bt_stop: TButton;
    Timer1: TTimer;
    PowerRange: TSpinEdit;
    LPowerRange: TLabel;
    Remanance: TSpinEdit;
    FreqPlage: TSpinEdit;
    LFreqPlage: TLabel;
    Seuil: TSpinEdit;
    EC2Z1LF1: TSpinEdit;
    EC2Z1LF2: TSpinEdit;
    EC2Z2LF1: TSpinEdit;
    EC2Z2LF2: TSpinEdit;
    Memo1: TMemo;
    PowerOffset: TSpinEdit;
    window: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Bt_startClick(Sender: TObject);
    procedure Bt_stopClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure PowerRangeChange(Sender: TObject);
    procedure CB_RemananceChange(Sender: TObject);
    procedure FreqPlageChange(Sender: TObject);
    procedure SeuilChange(Sender: TObject);
    procedure EC2Z1LF1Change(Sender: TObject);
    procedure EC2Z1LF2Change(Sender: TObject);
    procedure EC2Z2LF1Change(Sender: TObject);
    procedure EC2Z2LF2Change(Sender: TObject);
    procedure PowerOffsetChange(Sender: TObject);
    procedure windowChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

    //##########################################################################
    // Variables globales - begin
    //##########################################################################
    // Fentres
    bmFFT :CBM32 ;
    bmTri :CBM32 ;
    bmFen1 :CBM32 ;
    bmFen2 :CBM32 ;
    s_remanance : single ;
    Blue : TMyPixelFormat32bppARGB ;
    Freq_plage_dec0 : integer ;
    C2_Z1_LF1, C2_Z1_LF2, C2_Z1_F, C2_Z2_LF1, C2_Z2_LF2, C2_Z2_F :integer ;
    Son : PSon_S16 ;
    module_c, module_offset,  Seuil_bruit : double ;
    fps : string ;
    t, t0 : double ;
    FFT : CFFT_iterative ;
    Nb_image : integer ;
    rec : TWaveRecorder ;
    //##########################################################################
    // Variables globales - end
    //##########################################################################

    procedure Rec_Open( Sender: TObject );
    procedure Rec_Close( Sender: TObject );
    procedure Rec_Traitement( PSon :PSon_S16 ; TailleSon_Octet, Nb_Data : integer ; var TailleSon_Octet_Traite :integer )  ;
    procedure setCaption ;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  i :integer ;
  col : TMyPixelFormat32bppARGB ;
begin
  //############################################################################
  // Initialisation des variables globales - begin
  //############################################################################

  module_c := log10( 255 );
  module_offset := 0 ;

  GetMem( Son , c_Octets_Par_ValS16 * 2048 ) ;

  fps := '?' ;
  Blue.a := 255 ;
  Blue.c.r := 0 ;
  Blue.c.v := 0 ;
  Blue.c.b := 255 ;
  FFT := CFFT_iterative.create ;

  //gen_bit_reversal_4096 ;
  //gen_hamming_4096 ;
  //gen_hanning_4096 ;
  //gen_Blackman_4096 ;
  //gen_Blackman_Harris_4096 ;
  //gen_Blackman_Nuttall_4096 ;

  window.Items.Add( 'hamming'  );
  window.Items.Add( 'hanning'  );
  window.Items.Add( 'Blackman'  );
  window.Items.Add( 'Blackman_Harris'  );
  window.Items.Add( 'Blackman_Nuttall'  );
  window.ItemIndex := 0 ;
  FFT.window := @tab_hamming_4096 ;

  Nb_image := 0 ;
  bmFFT := CBM32.createLH( 1024, 256 ) ;
  bmTri := CBM32.createLH( 256, 256 )  ;
  bmFen1 :=  CBM32.createLH( 256, 256 )  ;
  bmFen2 :=  CBM32.createLH( 256, 256 )  ;

  col.a := 255 ;
  col.c.r := 0 ;
  col.c.v := 0 ;
  col.c.b := 0 ;

  bmFFT.Fill( col  );
  bmTri.Fill( col  );
  bmFen1.Fill( col  );
  bmFen2.Fill( col  );

  rec := TWaveRecorder.create( self ) ;
  rec.Parent := self ;
  rec.Nombre_Echantillon_Tampon := 2048 ;
  rec.TraiterDonnees := Rec_Traitement ;
  rec.OnOpen := Rec_Open ;
  rec.OnClose := Rec_Close ;

  memo1.lines.Add( 'Duree_Tampon = ' +formatfloat('0.00',rec.Duree_Tampon  )+' sec ' + formatfloat('0.00',1/rec.Duree_Tampon  ) +'(1/t)' );
  memo1.lines.Add( 'Images par seconde < ' +formatfloat('0.00',1/rec.Duree_Tampon  )+ ' (contrainte tampon)' );
  memo1.lines.Add( 'Liste des pilotes : ' ) ;
  memo1.lines.Add( rec.ListeDesPilotes.Text ) ;

  if rec.Nombre_Pilotes > 0 then
    begin
      for i := 0 to rec.ListeDesPilotes.count-1 do
        begin
          CB_Pilotes.Items.Add( rec.ListeDesPilotes.Strings[i] ) ;
        end ;
      CB_Pilotes.Enabled := true ;
      CB_Pilotes.ItemIndex := rec.Pilote ;
    end ;

  PowerOffsetChange( self );
  PowerRangeChange( self );
  CB_RemananceChange( self ) ;
  FreqPlageChange(Self);
  SeuilChange(Self) ;

  EC2Z1LF1Change( self ) ;
  EC2Z1LF2Change( self ) ;
  EC2Z2LF1Change( self ) ;
  EC2Z2LF2Change( self ) ;
  setCaption ;

  //############################################################################
  // Initialisation des variables globales - end
  //############################################################################
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  rec.Stopper ;
  rec.Free ;
  bmFen1.Free ;
  bmFen2.Free ;
  bmTri.Free ;
  bmFFT.Free ;
  FFT.Free ;
  FreeMem( Son  ) ;
end;

procedure TForm1.Rec_Open( Sender: TObject );
begin
end;

procedure TForm1.Rec_Close( Sender: TObject );
begin
end ;

function Decale( Freq_plage_dec0, val : integer ) : integer ;
begin
    if Freq_plage_dec0 < 0 then
      Result := val  shl (-Freq_plage_dec0)
    else
      if Freq_plage_dec0 > 0 then
        Result := val  shr (Freq_plage_dec0)
      else
        Result := val ;
end ;

procedure TForm1.Rec_Traitement( PSon :PSon_S16 ; TailleSon_Octet, Nb_Data : integer ; var TailleSon_Octet_Traite :integer )  ;
var
  i,idiv, ii, imod,   nb,addry, addr,  F, j, y, ym, ym0, daddr, i0 : integer ;
  Son : Tval_S16 ;
  encore : boolean ;
  module, fMax, C2_Pnt_x, C2_Pnt_y, C2_z1_m, C2_z2_m, fond, fondT  : double ;
  Pix : TMyPixelFormat32bppARGB ;
  scanTri, scanFFT, scanFen1, ScanFen2 : PMyTabPixelFormat32bppARGB ;
  PenGreen, PenRed, PenBlue, PenYellow :TGPPen ;
  grCanvas, GrFFT, GrTri, GrFen1, GrFen2 : TGPGRaphics ;
begin
  //############################################################################
  // init - begin
  //############################################################################
  grCanvas := TGPGRaphics.Create( canvas.Handle );
  grFFT := TGPGRaphics.Create( BMFFt.BM );
  grTri := TGPGRaphics.Create( BMTri.BM );
  grFen1 := TGPGRaphics.Create( BMFen1.BM );
  grFen2 := TGPGRaphics.Create( BMFen2.BM );
  scanFFT :=  BMFFT.data^.Scan0 ;
  scanTri :=  BMTri.data^.Scan0 ;
  scanFen1 :=  BMFen1.data^.Scan0 ;
  scanFen2 :=  BMFen2.data^.Scan0 ;
  PenGreen := TGPPen.Create(  MakeCOlor( 255, 0, 255 ,0) , 1 );
  PenRed := TGPPen.Create(  MakeCOlor( 128, 255, 0 ,0) , 1 );
  PenBlue := TGPPen.Create(  MakeCOlor( 128, 0, 0 ,255) , 1 );
  PenYellow := TGPPen.Create(  MakeCOlor( 255, 255, 255 ,0) , 1 );
  //############################################################################
  // init - end
  //############################################################################


  //############################################################################
  // Appel  FFT - begin
  //############################################################################
  FFT.FFT_2048_2048( PSon  );
  //############################################################################
  // Appel  FFT - end
  //############################################################################


  //############################################################################
  // Affichage du spectre - begin
  //############################################################################
  module :=  FFT.module[ 0 ]-module_offset ;
  if module < Seuil_bruit then module :=  0 ;
  ym := round(255 - ( module ) * module_c  )   ;
  if ym < 0  then ym := 0 ;
  i0 := 0 ;
  ym0 := ym ;

  for ii := 1 to 1023 do
  begin
    if Freq_plage_dec0 > 0 then
      i := ii  shl Freq_plage_dec0
    else
      if Freq_plage_dec0 < 0 then
        i := ii  shr (-Freq_plage_dec0)
      else
        i := ii ;

      module := FFT.module[ i ]- module_offset ;
      if module < Seuil_bruit then module :=  0 ;
      ym := round(255 - ( module  ) * module_c  )   ;
      if ym < 0 then ym := 0 ;
      GrFFT.DrawLine( PenGreen, i0,ym0,ii, ym    ) ;
      i0 := ii ;
      ym0 := ym ;
  end ;
  //############################################################################
  // Affichage du spectre - end
  //############################################################################


  //############################################################################
  // Recherche de C2_Z1_F & C2_Z2_F - begin
  //############################################################################
  FMax := -1e30 ;
  F := 0 ;
  C2_z1_m := 0 ;
  C2_Z1_F := -1 ;
  C2_Z2_F := -1 ;
  for i := C2_Z1_LF1 to C2_Z1_LF2 do
    begin
      module := FFT.module[ i ]-module_offset ;
      if Fmax < module then
        begin
          Fmax := module ;
          C2_z1_m := module ;
          F := i ;
        end ;
    end ;
  if FMax > Seuil_bruit then
    begin
      // recherchde de la fondamentale
      fondT := F ;
      fond  := F/2 ;
      encore := fond > C2_Z1_LF1 ;
      while encore do
        begin
          if FFT.module[ round( fond ) ]-module_offset > Seuil_bruit then
            begin
              fondT := Fond ;
            end ;
          fond  := fond/2 ;
          encore := fond > C2_Z1_LF1 ;
        end ;
      C2_Z1_F := round( fondT ) ;
      // recherchde de la fondamentale end
      FMax := -1e30 ;
      C2_z2_m := 0 ;
      for i := max(C2_Z2_LF1,C2_Z1_F+1) to C2_Z2_LF2 do
        begin
          module := FFT.module[ i ]-module_offset ;
          if Fmax < module then
            begin
              C2_z2_m := module ;
              Fmax := module ;
              F := i ;
            end ;
        end ;
      if FMax > Seuil_bruit then
        begin
      // recherchde de la fondamentale
      fondT := F ;
      fond  := F/2 ;
      encore := fond > C2_Z2_LF1 ;
      while encore do
        begin
          if FFT.module[ round( fond ) ]-module_offset > Seuil_bruit then
            begin
              fondT := Fond ;
            end ;
          fond  := fond/2 ;
          encore := fond > C2_Z2_LF1 ;
        end ;
      C2_Z2_F := round( fondT ) ;
      // recherchde de la fondamentale end
          F := Decale( Freq_plage_dec0, C2_Z1_F ) ;
          ym := round(255 - ( C2_z1_m - module_offset) * module_c  )   ;
          grFFT.DrawRectangle( PenRed, F-5 , ym-5, 10, 10 ) ;

          F := Decale( Freq_plage_dec0, C2_Z2_F ) ;
          ym := round(255 - ( C2_z2_m- module_offset ) * module_c  )   ;
          grFFT.DrawRectangle( PenBlue, F-5 , ym-5, 10, 10 ) ;

          C2_Pnt_x := round( (C2_Z1_F-C2_Z1_LF1)*255/(C2_Z1_LF2-C2_Z1_LF1) ) ;
          C2_Pnt_y := round( 255-(C2_Z2_F-C2_Z2_LF1)*255/(C2_Z2_LF2-C2_Z2_LF1) ) ;

          grTri.DrawRectangle( PenYellow, C2_Pnt_x-5 , C2_Pnt_y-5, 10, 10 ) ;
          grFen1.DrawRectangle( PenYellow, C2_Pnt_x-5 , C2_Pnt_y-5, 10, 10 ) ;
          grFen2.DrawRectangle( PenYellow, C2_Pnt_x-5 , C2_Pnt_y-5, 10, 10 ) ;

        end ;
    end ;
  //############################################################################
  // Recherche de C2_Z1_F & C2_Z2_F - end
  //############################################################################


  //############################################################################
  // Affichage des lignes sur le spectre - begin
  //############################################################################
  F := Decale( Freq_plage_dec0, C2_Z1_LF1 ) ;
  GrFFT.DrawLine( PenRed, F,0,F, 255   ) ;
  F := Decale( Freq_plage_dec0, C2_Z1_LF2 ) ;
  GrFFT.DrawLine( PenRed, F,0,F, 255   ) ;

  F := Decale( Freq_plage_dec0, C2_Z2_LF1 ) ;
  GrFFT.DrawLine( PenBlue, F,0,F, 255   ) ;
  F := Decale( Freq_plage_dec0, C2_Z2_LF2 ) ;
  GrFFT.DrawLine( PenBlue, F,0,F, 255   ) ;

  F := round(255 - 255 * (Seuil_bruit / module_c ) ) ;
  GrFFT.DrawLine( PenRed, 0,F,1024, F ) ;
  //############################################################################
  // Affichage des lignes sur le spectre - begin
  //############################################################################


  //############################################################################
  // Vers cran - begin
  //############################################################################
  self.Paint ;
  //############################################################################
  // Vers cran - end
  //############################################################################


  //############################################################################
  // Rmanance - begin
  //############################################################################
  for i := 0 to bmFFT.data^.Width*bmFFT.data^.Height -1 do
  begin
    Pix := ScanFFT^[ i ] ;
    Pix.c.r := round(Pix.c.r * s_remanance) ;
    Pix.c.v := round(Pix.c.v * s_remanance) ;
    Pix.c.b := round(Pix.c.b * s_remanance) ;
    ScanFFT^[ i ] :=  Pix ;
  end ;
  for i := 0 to bmTri.data^.Width*bmTri.data^.Height -1 do
  begin
    Pix := ScanTri^[ i ] ;
    Pix.c.r := round(Pix.c.r * s_remanance) ;
    Pix.c.v := round(Pix.c.v * s_remanance) ;
    Pix.c.b := round(Pix.c.b * s_remanance) ;
    ScanTri^[ i ] :=  Pix ;
  end ;
  for i := 0 to bmFen1.data^.Width*bmFen1.data^.Height -1 do
  begin
    Pix := ScanFen1^[ i ] ;
    Pix.c.r := round(Pix.c.r * s_remanance) ;
    Pix.c.v := round(Pix.c.v * s_remanance) ;
    Pix.c.b := round(Pix.c.b * s_remanance) ;
    ScanFen1^[ i ] :=  Pix ;
  end ;
  for i := 0 to bmFen2.data^.Width*bmFen2.data^.Height -1 do
  begin
    Pix := ScanFen2^[ i ] ;
    Pix.c.r := round(Pix.c.r * s_remanance) ;
    Pix.c.v := round(Pix.c.v * s_remanance) ;
    Pix.c.b := round(Pix.c.b * s_remanance) ;
    ScanFen2^[ i ] :=  Pix ;
  end ;
  //############################################################################
  // Rmanance - end
  //############################################################################


  //############################################################################
  // Images/s - begin
  //############################################################################
  inc( Nb_image ) ;
  if nb_image > 80 then
    begin
      t := time ;
      if t > t0 then
        fps := formatfloat('0.00', Nb_image / ( (t-t0)*(3600*24)) )
      else
        fps := '?' ;
      t0 :=  t ;
      nb_image  := 0 ;
      setCaption ;
      Application.ProcessMessages ;
    end ;
  //############################################################################
  // Images/s - end
  //############################################################################

  TailleSon_Octet_Traite :=  Nb_Data ;

  //############################################################################
  // Destruction - begin
  //############################################################################
  PenYellow.Free ;
  PenBlue.Free ;
  PenGreen.Free ;
  GrFen1.Free ;
  GrFen2.Free ;
  PenRed.Free ;
  grTri.free ;
  GrFFT.Free ;
  grCanvas.Free ;
  //############################################################################
  // Destruction - end
  //############################################################################
end ;


procedure TForm1.Bt_startClick(Sender: TObject);
begin
  rec.Pilote := CB_Pilotes.ItemIndex ;
  t := time ;
  t0 :=  t ;
  fps := '?' ;
  FFT.init ;
  rec.Enrgistrer ;
end;

procedure TForm1.Bt_stopClick(Sender: TObject);
begin
  fps := '?' ;
  setCaption ;
  rec.Stopper ;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  Gr : TGPGraphics ;
begin
  Gr := TGPGraphics.Create( canvas.Handle )  ;
  gr.DrawImage(  BMFFT.BM , 0 ,0 ) ;
  gr.DrawImage(  BMTri.BM , 241 ,257 ) ;
  gr.DrawImage(  BMFen1.BM , 241+257 ,257 ) ;
  gr.DrawImage(  BMFen2.BM , 241+257+257 ,257 ) ;

  Gr.Free ;
end;

procedure TForm1.setCaption ;
begin
  caption := Titre + fps + ' img/s' ;
end ;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  i, nt : integer ;
begin
  for i := 0 to 1013 do
    begin
      Son^[i].g := round(
                      cos( 2 * pi * (1000) * (i/44100)+ 0.5+nb_image  ) * 6000 +
                      cos( 2 * pi * (5000) * (i/44100)+ 0.5+nb_image  ) * 6000 +
                      cos( 2 * pi * (10000) * (i/44100)+ 0.1+nb_image ) * 6000 +
                      sin( 2 * pi * (15000) * (i/44100)+0.3+nb_image  ) * 6000);
      Son^[i].d := 0 ;
    end ;
  self.Rec_Traitement( Son, 1024*c_Octets_Par_ValS16 , 1024, nt );
end;


procedure TForm1.PowerOffsetChange(Sender: TObject);
begin
  try
      module_offset := PowerOffset.value    ;
  except
  end ;
  LPowerRange.Caption := formatfloat('0.000' ,module_offset)+' to ' + formatfloat('0.000 dB', 255 / module_c+module_offset ) ;
end;

procedure TForm1.PowerRangeChange(Sender: TObject);
var
  db : double ;
begin
  db := PowerRange.Value    ;
  try
  if PowerRange.Value > 0 then
    begin
      module_c := 255 /  PowerRange.Value   ;
    end
  else
    begin
      module_c := 255 * ( -PowerRange.Value+1)   ;
    end
  except
  end ;

  LPowerRange.Caption := formatfloat('0.000' ,module_offset)+' to ' + formatfloat('0.000 dB', db + module_offset ) ;
end;

procedure TForm1.CB_RemananceChange(Sender: TObject);
begin
  try
      s_remanance := remanance_defaut[ Remanance.value ]   ;
  except
  end ;
end;

procedure TForm1.FreqPlageChange(Sender: TObject);
begin
  try
      Freq_plage_dec0 := Freq_Plage_dec[ FreqPlage.value ]   ;
  except
  end ;
  LFreqPlage.Caption := '0 to ' + formatfloat('0,000. Hz', Freq_Plage_val[ FreqPlage.value ] ) ;
end;

procedure TForm1.SeuilChange(Sender: TObject);
begin
  try
    Seuil_bruit := module_c * seuil.value / 100 ;
  except
  end ;
end;

procedure TForm1.EC2Z1LF1Change(Sender: TObject);
begin
  try
    C2_Z1_LF1 := round(4096*EC2Z1LF1.Value/44100) ;
  except
  end ;
end;

procedure TForm1.EC2Z1LF2Change(Sender: TObject);
begin
  try
    C2_Z1_LF2 := round(4096*EC2Z1LF2.Value/44100) ;
  except
  end ;
end;

procedure TForm1.EC2Z2LF1Change(Sender: TObject);
begin
  try
    C2_Z2_LF1 := round(4096*EC2Z2LF1.Value/44100) ;
  except
  end ;
end;

procedure TForm1.EC2Z2LF2Change(Sender: TObject);
begin
  try
    C2_Z2_LF2 := round(4096*EC2Z2LF2.Value/44100) ;
  except
  end ;
end;

procedure TForm1.windowChange(Sender: TObject);
begin
  case window.ItemIndex of
    0 : FFT.window := @tab_hamming_4096 ;
    1 : FFT.window := @tab_hanning_4096 ;
    2 : FFT.window := @tab_Blackman_4096 ;
    3 : FFT.window := @tab_Blackman_Harris_4096 ;
    4 : FFT.window := @tab_Blackman_Nuttall_4096 ;
    else
      FFT.window := @tab_hamming_4096 ;
  end;
end;

end.
