{
 ***** BEGIN LICENSE BLOCK *****
 The contents of this file are subject to the Mozilla Public
 License Version 1.1 (the "License"); you may not use this file
 except in compliance with the License. You may obtain a copy of
 the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS
 IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 implied. See the License for the specific language governing
 rights and limitations under the License.

 The Original Code is AnalyseSpectrale code.

 The Initial Developer of the Original Code is Arthur Torossian.
 Portions created by Arthur Torossian are Copyright (C) 2008 Arthur Torossian,
 living in Paris, 20 rue Sibuet, in 2008. All Rights Reserved.

 Original Author: Arthur Torossian (arthur.torossian@free.fr)

 Contributor(s): None

 Alternatively, the contents of this file may be used under the terms of the
 GNU Lesser General Public License (the  "LGPL License"), in which case the
 provisions of the LGPL License are applicable instead of those above.
 If you wish to allow use of your version of this file only under the terms
 of the LGPL License and not to allow others to use your version of this file
 under the MPL, indicate your decision by deleting  the provisions above and
 replace  them with the notice and other provisions required by the LGPL
 License.  If you do not delete the provisions above, a recipient may use
 your version of this file under either the MPL or the LGPL License.

 For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html
 ***** END LICENSE BLOCK ***** */
}

unit UTools;

interface
uses
  windows
  , Graphics
  , SysUtils
  , classes
  , GDIPAPI
  , GDIPOBJ
  ;
type
  MyStatus = (
    Ok,
    GenericError,
    InvalidParameter,
    OutOfMemory,
    ObjectBusy,
    InsufficientBuffer,
    NotImplemented,
    Win32Error,
    WrongState,
    Aborted,
    FileNotFound,
    ValueOverflow,
    AccessDenied,
    UnknownImageFormat,
    FontFamilyNotFound,
    FontStyleNotFound,
    NotTrueTypeFont,
    UnsupportedGdiplusVersion,
    GdiplusNotInitialized,
    PropertyNotFound,
    PropertyNotSupported
  );

  TComplex = record re, im : double ; end ;

  TMyPixelFormat24bppRGB  =
    packed record
      b  : Byte;
      v  : Byte;
      r  : Byte;
    end;


  TMyPixelFormat32bppARGB  =
    packed record
      c : TMyPixelFormat24bppRGB ;
      a : Byte;
    end;

  TMyTabPixelFormat32bppARGB = packed array[ 0..100000 ] of TMyPixelFormat32bppARGB ;
  PMyTabPixelFormat32bppARGB = ^TMyTabPixelFormat32bppARGB ;

  TMyBitmapDataPixelFormat32bppARGB =
    packed record
      Width       : UINT;
      Height      : UINT;
      Stride      : longint;
      PixelFormat : TPixelFormat ;
      Scan0       : PMyTabPixelFormat32bppARGB ;
      Reserved    : UINT;
    end;
  PMyBitmapDataPixelFormat32bppARGB = ^TMyBitmapDataPixelFormat32bppARGB ;
type
  CBM32 =
    class
      BM :TGPBitMap ;
      data : PMyBitmapDataPixelFormat32bppARGB ;
      constructor createLH( l,h :UINT ) ; overload ;  Virtual  ;
      destructor destroy ; override  ;
      procedure  Fill( col  : TMyPixelFormat32bppARGB ) ;
    end ;

function   CreerTGPBitmap( D :PMyBitmapDataPixelFormat32bppARGB ):TGPBitmap ;
function  Free_PMyBitmapDataPixelFormat32bppARGB(  var d :PMyBitmapDataPixelFormat32bppARGB ; PixelsToDeReserve : boolean = true ) :Mystatus;
function  Create_PMyBitmapDataPixelFormat32bppARGB( var d :PMyBitmapDataPixelFormat32bppARGB; l,h :UINT ; PixelsToReserve : boolean = true ) :Mystatus ;

implementation

constructor CBM32.createLH( l,h :UINT ) ;
begin
  inherited create ;
  Create_PMyBitmapDataPixelFormat32bppARGB( Data, l , h ) ;
  BM := CreerTGPBitmap( Data ) ;
end ;

procedure CBM32.Fill( col  : TMyPixelFormat32bppARGB ) ;
var
  n, i :longint ;
begin
  n := data.Width * data.height - 1 ;
  for i := 0 to n do
    begin
      data.scan0[i] := col ;
    end ;
end ;


destructor CBM32.destroy ;
begin
  Bm.Free ;
  Free_PMyBitmapDataPixelFormat32bppARGB( data ) ;
  inherited destroy ;
end ;

function Create_PMyBitmapDataPixelFormat32bppARGB( var d :PMyBitmapDataPixelFormat32bppARGB; l, h :UINT ; PixelsToReserve : boolean = true ) :Mystatus;
begin
  try

    GetMem( D , sizeof( D^ ) ) ;
    D.Width := L ;
    D.Height := H ;
    D.PixelFormat := PixelFormat32bppARGB ;
    //D.PixelFormat := PixelFormat32bppPARGB ;
    D.Stride := L * Sizeof( TMyPixelFormat32bppARGB ) ;
    try
      if PixelsToReserve then
        Getmem( D.Scan0 , L * H * Sizeof( TMyPixelFormat32bppARGB ) )
      else
        D.Scan0 := nil ;
      result := Ok ;
   except
      FreeMem( D ) ;
      D := nil ;
      result := OutOfMemory ;
    end ;
  except
    D := nil ;
    result := OutOfMemory ;
  end ;
end ;

function  CreerTGPBitmap( D :PMyBitmapDataPixelFormat32bppARGB ):TGPBitmap ;
begin
  Result := TGPBitmap.Create( d^.width, d^.height, d^.stride , d^.PixelFormat, PByte(d^.scan0) );
end ;

function Free_PMyBitmapDataPixelFormat32bppARGB(  var d :PMyBitmapDataPixelFormat32bppARGB ; PixelsToDeReserve : boolean = true ) :Mystatus;
begin
  if d <> nil then
    begin
      if PixelsToDeReserve then
        begin
          freeMem( d.Scan0 ) ;
        end ;
      freeMem( d  ) ;
    end ;
  d := nil ;
  result := Ok ;
end ;

end.
