{
 ***** BEGIN LICENSE BLOCK *****
 The contents of this file are subject to the Mozilla Public
 License Version 1.1 (the "License"); you may not use this file
 except in compliance with the License. You may obtain a copy of
 the License at http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS
 IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 implied. See the License for the specific language governing
 rights and limitations under the License.

 The Original Code is AnalyseSpectrale code.

 The Initial Developer of the Original Code is Arthur Torossian.
 Portions created by Arthur Torossian are Copyright (C) 2008 Arthur Torossian,
 living in Paris, 20 rue Sibuet, in 2008. All Rights Reserved.

 Original Author: Arthur Torossian (arthur.torossian@free.fr)

 Contributor(s): None

 Alternatively, the contents of this file may be used under the terms of the
 GNU Lesser General Public License (the  "LGPL License"), in which case the
 provisions of the LGPL License are applicable instead of those above.
 If you wish to allow use of your version of this file only under the terms
 of the LGPL License and not to allow others to use your version of this file
 under the MPL, indicate your decision by deleting  the provisions above and
 replace  them with the notice and other provisions required by the LGPL
 License.  If you do not delete the provisions above, a recipient may use
 your version of this file under either the MPL or the LGPL License.

 For more information about the LGPL: http://www.gnu.org/copyleft/lesser.html
 ***** END LICENSE BLOCK ***** */
}

unit UWaveRecorder;
interface
uses
  Windows,
  controls,
  Messages,
  mmSystem,
  SysUtils,
  classes,
  dialogs,
  UMyMMSystem ;
Type
  TWaveRecorder =
    class( TCustomControl )
      private
        uDeviceID: UINT;
        hWaveIn: HWaveIn ;
        Format: TWaveFormatEx ;
        //----------------------------------------------------------------------
        m_Ouvert,
        Capacite :boolean ;
        //----------------------------------------------------------------------
        m_t0 :double ;
        //----------------------------------------------------------------------
        Ordre_Stop :boolean ;
        WaveHDR1,
        WaveHDR2 :TWaveHDR ;
        m_Nb_Valeurs :integer ;
        m_ListeDesPilotes :TStringList ;
        //----------------------------------------------------------------------
        procedure AllouerTamponBloc ;
        procedure LibererTamponBloc ;
        function ValideRead : boolean ;
        procedure Pilote_Write( i : integer ) ;
        function Pilote_Read :integer ;
        function Position_Read: Cardinal  ;
        function Taille_Tampon_octet_Read :integer ;
        function Duree_Tampon_read :double ;
        function Nombre_Echantillon_Tampon_Read : integer ;
        procedure Nombre_Echantillon_Tampon_Write( Nb :integer ) ;
        function Pas_Echantillonnage_read :double ;
        function Nombre_Pilotes_Read :integer ;

      protected
        procedure MyWindowProc(var Message: TMessage);
      public
        OnOpen  : TNotifyEvent ;
        OnClose : TNotifyEvent ;
        TraiterDonnees : TTraiterData_S16 ;

        constructor Create(AOwner: TComponent); override ;
        destructor  destroy ; override ;
        function    Enrgistrer :boolean;
        procedure   Stopper ;
        property    Valide :boolean  read ValideRead ;
        property    Ouvert :boolean read m_ouvert ;
        property    ListeDesPilotes :TStringList read m_ListeDesPilotes ;
        property    Pilote : integer read Pilote_Read write Pilote_Write ;
        property    Position : Cardinal read Position_Read ;
        property    Taille_Tampon_octet :integer read Taille_Tampon_octet_Read ;
        property    Pas_Echantillonnage : double read  Pas_Echantillonnage_read ;
        property    Nombre_Pilotes :integer read Nombre_Pilotes_Read  ;
        property    Duree_Tampon :double  read Duree_Tampon_read ;
        property    Nombre_Echantillon_Tampon : integer read Nombre_Echantillon_Tampon_Read write Nombre_Echantillon_Tampon_Write ;
    end ;
implementation

procedure TWaveRecorder.MyWindowProc(var Message: TMessage);
var
  Writed :integer ;
begin
  case Message.Msg of
    WIM_OPEN :
      begin
        m_Ouvert := True ;
        if Assigned(OnOpen) then OnOpen( Self );
      end ;
    WIM_CLOSE :
      begin
        waveInUnPrepareHeader( hWaveIn, @WaveHDR1 , sizeof( TWaveHDR ) ) ;
        waveInUnPrepareHeader( hWaveIn, @WaveHDR2 , sizeof( TWaveHDR ) ) ;
        //----------------------------------------------------------------------
        if Assigned(OnClose) then OnClose( Self );
        //----------------------------------------------------------------------
        m_Ouvert := false ;
      end ;
    WIM_DATA :
      begin
        //----------------------------------------------------------------------
        if Assigned(TraiterDonnees) then TraiterDonnees( PSon_S16(PWaveHdr(Message.LParam)^.lpData),
                                                         PWaveHdr(Message.LParam)^.dwBytesRecorded,
                                                         PWaveHdr(Message.LParam)^.dwBytesRecorded div c_Octets_Par_ValS16,
                                                         writed ) ;
        PWaveHdr(Message.LParam)^.dwBytesRecorded := 0 ;
        //----------------------------------------------------------------------
        if not Ordre_Stop then
          begin
            waveInAddBuffer( Message.WParam, PWaveHdr(Message.LParam) , sizeof( TWaveHDR ) ) ;
          end
        else
          begin
          end ;
      end ;
    else
      WndProc(Message);
  end ;
end ;

function TWaveRecorder.Enrgistrer:boolean ;
begin
  Result := false ;
  if not Valide then exit ;
  if m_Ouvert then exit ;
      AllouerTamponBloc ;
      Ordre_Stop := false ;

      Format.wFormatTag :=  WAVE_FORMAT_PCM ;
      Format.nChannels := 2 ;
      Format.wBitsPerSample :=  16 ;

      Format.nBlockAlign := (Format.nChannels * Format.wBitsPerSample) div 8 ;
      Format.nAvgBytesPerSec := Format.nSamplesPerSec * Format.nBlockAlign ;
      Format.cbSize := 0 ;
      if waveInOpen( @hWavein, uDeviceID, @Format, Handle, 0, CALLBACK_WINDOW) <> MMSYSERR_NOERROR then exit ;
      WaveHDR1.dwFlags := 0 ;
          WaveHDR1.dwLoops := 0 ;
          WaveHDR1.Owner := Self ;
          WaveHDR1.dwBytesRecorded := 0 ;
      WaveHDR2.dwFlags := 0 ;
          WaveHDR2.dwLoops := 0 ;
          WaveHDR2.Owner := Self ;
          WaveHDR2.dwBytesRecorded := 0 ;
      if waveInPrepareHeader( hWaveIn, @WaveHDR1 , sizeof( WaveHDr1 ) )  <> MMSYSERR_NOERROR then exit ;
      if waveInAddBuffer( hWaveIn, @WaveHDR1, sizeof(WaveHDR1) )  <> MMSYSERR_NOERROR then exit ;
      if waveInPrepareHeader( hWaveIn, @WaveHDR2 , sizeof( WaveHDr2 ) )  <> MMSYSERR_NOERROR then exit ;
      if waveInAddBuffer( hWaveIn, @WaveHDR2, sizeof(WaveHDR2) )  <> MMSYSERR_NOERROR then exit ;
      if WaveInStart( hWaveIn )  <> MMSYSERR_NOERROR then exit ;
  Result := true ;
end ;

procedure TWaveRecorder.Stopper ;
begin
  if not Valide then exit ;
  if not m_ouvert then exit ;
  Ordre_Stop := true ;
  waveInStop( hWavein );
  waveInReset( hWavein );
  waveInClose( hWavein );
end ;

constructor TWaveRecorder.Create(AOwner: TComponent);
var
  Nb_Pilote_Wave :integer ;
  Caps: TWaveInCaps ;
  i :integer ;
  encore :boolean ;
  s :string ;
begin
  inherited create( AOwner ) ;
  Format.nSamplesPerSec := 44100 ; m_T0 := 1/Format.nSamplesPerSec ;
  m_ListeDesPilotes := TStringList.create ;
  WindowProc := MyWindowProc ;
  m_Ouvert := false ;
  Capacite := false ;
  WaveHDR1.lpNext := @WaveHDR2 ;
  WaveHDR2.lpNext := @WaveHDR1 ;
  m_Nb_Valeurs := c_Nb_Valeurs_Tampon_def ;
  Nb_Pilote_Wave := waveInGetNumDevs ;
  i := 0 ; encore := true ;
  while (i < Nb_Pilote_Wave) and encore do
    begin
      if waveInGetDevCaps(i, @Caps, sizeof( Caps )) = MMSYSERR_NOERROR then
        begin
          with Caps do
            begin
              //-----------------------------------------------------------
              if (dwFormats and WAVE_FORMAT_4S16) > 0 then
                begin
                  s := inttostr(i)+' - '+szPname ;
                  s := s + ' v'+inttostr( vDriverVersion  shr 8 )+'.'+ inttostr( vDriverVersion  and $F );
                  uDeviceID := i ;
                  Capacite := true ;
                  m_ListeDesPilotes.Add( s ) ;
                end ;
              //-----------------------------------------------------------
            end ;
        end ;
      inc(i);
    end ;
end ;

destructor TWaveRecorder.destroy ;
begin
  m_ListeDesPilotes.free ;
  LibererTamponBloc ;
  inherited destroy ;
end ;

function TWaveRecorder.ValideRead : boolean ;
begin
  Result := Capacite and ( Parent <> Nil ) ;
end ;

procedure TWaveRecorder.LibererTamponBloc ;
begin
  if WaveHdr1.LpData <> Nil then FreeMem( WaveHdr1.LpData ) ; WaveHdr1.LpData := Nil ;
  if WaveHdr2.LpData <> Nil then FreeMem( WaveHdr2.LpData ) ; WaveHdr2.LpData := Nil ;
  WaveHdr1.dwBufferLength := 0 ;
  WaveHdr2.dwBufferLength := 0 ;
end ;

function TWaveRecorder.Taille_Tampon_Octet_read :integer ;
begin
  Result := c_Octets_Par_ValS16 * m_nb_Valeurs ;
end ;

procedure TWaveRecorder.AllouerTamponBloc ;
begin
  LibererTamponBloc ;

  GetMem( WaveHdr1.LpData , Taille_Tampon_octet ) ;
  GetMem( WaveHdr2.LpData , Taille_Tampon_octet  ) ;

  WaveHdr1.dwBufferLength := Taille_Tampon_octet   ;
  WaveHdr2.dwBufferLength := Taille_Tampon_octet   ;
end ;

function TWaveRecorder.Position_Read: cardinal;
var
  T : MMTime ;
begin
  if m_Ouvert then
    begin
      T.wType := TIME_SAMPLES ;
      waveInGetPosition( HWaveIn , @T, Sizeof( MMTime )) ;
      if T.wType <> TIME_SAMPLES then
        begin
          Result := 0 ;
        end
      else
        begin
          Result := round(T.sample * m_T0 ) ;
        end ;
    end
  else
    begin
      Result := 0 ;
    end ;
end ;

procedure TWaveRecorder.Pilote_Write( i : integer ) ;
begin
  if (i>=0) and ( i < m_ListeDesPilotes.Count ) and ( not Ouvert ) then
    begin
      uDeviceID := i ;
    end ;
end ;

function TWaveRecorder.Nombre_Pilotes_Read :integer ;
begin
  Result := m_ListeDesPilotes.Count ;
end ;

function TWaveRecorder.Pilote_Read :integer ;
begin
  Result := uDeviceID ;
  if Not Capacite then Result := -1 ;
end ;

function TWaveRecorder.Pas_Echantillonnage_read :double ;
begin
  Result := m_t0 ;
end ;

function TWaveRecorder.Duree_Tampon_read :double ;
begin
  Result := m_Nb_Valeurs * m_t0 ;
end ;

function TWaveRecorder.Nombre_Echantillon_Tampon_Read : integer ;
begin
  Result := m_Nb_Valeurs ;
end ;

procedure TWaveRecorder.Nombre_Echantillon_Tampon_Write( Nb :integer ) ;
begin
  if not ouvert then
    begin
      m_Nb_Valeurs := Nb ;
    end ;
end ;

end.
